@@ -3,7 +3,7 @@
   array.c -
 
   $Author: matz $
-  $Date: 1999/01/21 09:04:50 $
+  $Date: 1999/02/09 03:29:20 $
   created at: Fri Aug  6 09:46:12 JST 1993
 
   Copyright (C) 1993-1998 Yukihiro Matsumoto
@@ -864,7 +864,7 @@ ary_delete_if(ary)
 
     ary_modify(ary);
     for (i1 = i2 = 0; i1 < RARRAY(ary)->len; i1++) {
-	if (rb_yield(RARRAY(ary)->ptr[i1])) continue;
+	if (RTEST(rb_yield(RARRAY(ary)->ptr[i1]))) continue;
 	if (i1 != i2) {
 	    RARRAY(ary)->ptr[i2] = RARRAY(ary)->ptr[i1];
 	}
