@@ -3,7 +3,7 @@
   bignum.c -
 
   $Author: matz $
-  $Date: 1999/01/11 07:55:24 $
+  $Date: 1999/02/09 03:29:20 $
   created at: Fri Jun 10 00:48:55 JST 1994
 
 ************************************************/
@@ -395,7 +395,7 @@ dbl2big(d)
     VALUE z;
     double u = (d < 0)?-d:d;
 
-    while (0 != (long)u) {
+    while (!FIXABLE(u) || 0 != (long)u) {
 	u /= (double)(BIGRAD);
 	i++;
     }
