@@ -3,7 +3,7 @@
   dln.c -
 
   $Author: matz $
-  $Date: 1998/12/14 09:00:05 $
+  $Date: 1999/02/09 03:29:21 $
   created at: Tue Jan 18 17:05:06 JST 1994
 
   Copyright (C) 1993-1998 Yukihiro Matsumoto
@@ -78,7 +78,7 @@ int eaccess();
 #endif
 
 #ifndef FUNCNAME_PATTERN
-# if defined(__hp9000s300) || defined(__NetBSD__) || defined(__BORLANDC__) || (defined(__FreeBSD__) && __FreeBSD__ < 3) || defined(NeXT) || defined(__WATCOMC__)
+# if defined(__hp9000s300) ||  ( defined(__NetBSD__) && (!defined(__alpha__) && !defined(__mips__))) || defined(__BORLANDC__) || (defined(__FreeBSD__) && __FreeBSD__ < 3) || defined(NeXT) || defined(__WATCOMC__)
 #  define FUNCNAME_PATTERN "_Init_%.200s"
 # else
 #  define FUNCNAME_PATTERN "Init_%.200s"
