@@ -3,7 +3,7 @@
   pack.c -
 
   $Author: matz $
-  $Date: 1998/12/25 04:35:41 $
+  $Date: 1999/01/18 09:08:50 $
   created at: Thu Feb 10 15:17:05 JST 1994
 
   Copyright (C) 1993-1998 Yukihiro Matsumoto
@@ -134,7 +134,7 @@ pack_pack(ary, fmt)
 	  case 'H': case 'h':
 	    from = NEXTFROM;
 	    if (NIL_P(from)) {
-		ptr = 0;
+		ptr = "";
 		plen = 0;
 	    }
 	    else {
