@@ -3,7 +3,7 @@
   ruby.c -
 
   $Author: matz $
-  $Date: 1998/12/14 09:00:07 $
+  $Date: 1999/01/21 09:04:56 $
   created at: Tue Aug 10 12:47:31 JST 1993
 
   Copyright (C) 1993-1998 Yukihiro Matsumoto
@@ -715,10 +715,15 @@ ruby_libpath()
     strcpy(libpath, __dos_argv0);
 #endif
     p = strrchr(libpath, '\\');
-    if (p)
+    if (p) {
 	*p = 0;
-    if (!strcasecmp(p-4, "\\bin"))
-	p -= 4;
+	if (!strcasecmp(p-4, "\\bin"))
+	    p -= 4;
+    } else {
+	strcpy(libpath, ".");
+	p = libpath + 1;
+    }
+
     strcpy(p, "\\lib");
 #if defined(__CYGWIN32__)
     p = (char *)malloc(strlen(libpath)+10);
