@@ -24,63 +24,81 @@ end
 
 # make sure conditional operators work
 
+check "assignment"
+
+a=[]; a[0] ||= "bar";
+ok(a[0] == "bar")
+h={}; h["foo"] ||= "bar";
+ok(h["foo"] == "bar")
+
+aa = 5
+aa ||= 25
+ok(aa == 5)
+bb ||= 25
+ok(bb == 25)
+cc &&=33
+ok(cc == nil)
+cc = 5
+cc &&=44
+ok(cc == 44)
+
 check "condition"
 
 $x = '0';
 
-$x == $x && ok(TRUE)
-$x != $x && ok(FALSE)
-$x == $x || ok(FALSE)
-$x != $x || ok(TRUE)
+$x == $x && ok(true)
+$x != $x && ok(false)
+$x == $x || ok(false)
+$x != $x || ok(true)
 
 # first test to see if we can run the tests.
 
 check "if/unless";
 
 $x = 'test';
-ok(if $x == $x then TRUE else FALSE end)
-$bad = FALSE
+ok(if $x == $x then true else false end)
+$bad = false
 unless $x == $x
-  $bad = TRUE
+  $bad = true
 end
 ok(!$bad)
-ok(unless $x != $x then TRUE else FALSE end)
+ok(unless $x != $x then true else false end)
 
 check "case"
 
 case 5
 when 1, 2, 3, 4, 6, 7, 8
-  ok(FALSE)
+  ok(false)
 when 5
-  ok(TRUE)
+  ok(true)
 end
 
 case 5
 when 5
-  ok(TRUE)
+  ok(true)
 when 1..10
-  ok(FALSE)
+  ok(false)
 end
 
 case 5
 when 1..10
-  ok(TRUE)
+  ok(true)
 else
-  ok(FALSE)
+  ok(false)
 end
 
 case 5
 when 5
-  ok(TRUE)
+  ok(true)
 else
-  ok(FALSE)
+  ok(false)
 end
 
 case "foobar"
 when /^f.*r$/
-  ok(TRUE)
+  ok(true)
 else
-  ok(FALSE)
+  ok(false)
 end
 
 check "while/until";
@@ -106,17 +124,17 @@ ok(!tmp.eof? && /vt100/)
 tmp.close
 
 # test next
-$bad = FALSE
+$bad = false
 tmp = open("while_tmp", "r")
 while tmp.gets()
-    next if /vt100/;
-    $bad = 1 if /vt100/;
+  next if /vt100/;
+  $bad = 1 if /vt100/;
 end
 ok(!(!tmp.eof? || /vt100/ || $bad))
 tmp.close
 
 # test redo
-$bad = FALSE
+$bad = false
 tmp = open("while_tmp", "r")
 while tmp.gets()
   if gsub!('vt100', 'VT100')
@@ -140,12 +158,12 @@ end
 ok(sum == 220)
 
 # test interval
-$bad = FALSE
+$bad = false
 tmp = open("while_tmp", "r")
 while tmp.gets()
   break unless 1..2
   if /vt100/ || /Amiga/ || /paper/
-    $bad = TRUE
+    $bad = true
     break
   end
 end
@@ -166,22 +184,22 @@ check "exception";
 
 begin
   fail "this must be handled"
-  ok(FALSE)
+  ok(false)
 rescue
-  ok(TRUE)
+  ok(true)
 end
 
-$bad = TRUE
+$bad = true
 begin
   fail "this must be handled no.2"
 rescue
   if $bad
-    $bad = FALSE
+    $bad = false
     retry
-    ok(FALSE)
+    ok(false)
   end
 end
-ok(TRUE)
+ok(true)
 
 # exception in rescue clause
 $string = "this must be handled no.3"
@@ -191,9 +209,9 @@ begin
   rescue 
     fail $string
   end
-  ok(FALSE)
+  ok(false)
 rescue
-  ok(TRUE) if $! == $string
+  ok(true) if $! == $string
 end
   
 # exception in ensure clause
@@ -203,43 +221,55 @@ begin
   ensure 
     fail "exception in ensure clause"
   end
-  ok(FALSE)
+  ok(false)
 rescue
-  ok(TRUE)
+  ok(true)
 end
 
-$bad = TRUE
+$bad = true
 begin
   begin
     fail "this must be handled no.5"
   ensure
-    $bad = FALSE
+    $bad = false
   end
 rescue
 end
 ok(!$bad)
 
-$bad = TRUE
+$bad = true
 begin
   begin
     fail "this must be handled no.6"
   ensure
-    $bad = FALSE
+    $bad = false
   end
 rescue
 end
 ok(!$bad)
 
-$bad = TRUE
-while TRUE
+$bad = true
+while true
   begin
     break
   ensure
-    $bad = FALSE
+    $bad = false
   end
 end
 ok(!$bad)
 
+ok(catch(:foo) {
+     loop do
+       loop do
+	 throw :foo, true
+	 break
+       end
+       break
+       ok(false)			# should no reach here
+     end
+     false
+   })
+
 check "array"
 ok([1, 2] + [3, 4] == [1, 2, 3, 4])
 ok([1, 2] * 2 == [1, 2, 1, 2])
@@ -265,11 +295,20 @@ ok($x[0] == -1 && $x[1] == 10)
 $x[-1, 1] = 20
 ok($x[-1] == 20 && $x.pop == 20)
 
+# array and/or
+ok(([1,2,3]&[2,4,6]) == [2])
+ok(([1,2,3]|[2,4,6]) == [1,2,3,4,6])
+
 # compact
 $x = [nil, 1, nil, nil, 5, nil, nil]
 $x.compact!
 ok($x == [1, 5])
 
+# uniq
+$x = [1, 1, 4, 2, 5, 4, 5, 1, 2]
+$x.uniq!
+ok($x == [1, 4, 2, 5])
+
 # empty?
 ok(!$x.empty?)
 $x = []
@@ -287,11 +326,17 @@ ok($x == [7,5,3,2,1])
 
 # split test
 $x = "The Book of Mormon"
-ok($x.split(//).reverse!.join == "nomroM fo kooB ehT")
+ok($x.split(//).reverse!.join == $x.reverse)
+ok($x.reverse == $x.reverse!)
 ok("1 byte string".split(//).reverse.join(":") == "g:n:i:r:t:s: :e:t:y:b: :1")
 $x = "a b c  d"
 ok($x.split == ['a', 'b', 'c', 'd'])
 ok($x.split(' ') == ['a', 'b', 'c', 'd'])
+ok(defined? "a".chomp)
+ok("abc".scan(/./) == ["a", "b", "c"])
+ok("1a2b3c".scan(/(\d.)/) == [["1a"], ["2b"], ["3c"]])
+# non-greedy match
+ok("a=12;b=22".scan(/(.*?)=(\d*);?/) == [["a", "12"], ["b", "22"]])
 
 $x = [1]
 ok(($x * 5).join(":") == '1:1:1:1:1')
@@ -312,16 +357,16 @@ ok(begin
      for k,v in $y
        fail if k*2 != v
      end
-     TRUE
+     true
    rescue
-     FALSE
+     false
    end)
 
 ok($x.length == 3)
 ok($x.has_key?(1))
 ok($x.has_value?(4))
 ok($x.indexes(2,3) == [4,6])
-ok($x == (1=>2, 2=>4, 3=>6))
+ok($x == {1=>2, 2=>4, 3=>6})
 
 $z = $y.keys.join(":")
 ok($z == "1:2:3")
@@ -371,39 +416,39 @@ ok(i == 5)
 # iterator break/redo/next/retry
 unless defined? loop
   def loop
-    while TRUE
+    while true
       yield
     end
   end
-  ok(FALSE)
+  ok(false)
 else
-  ok(TRUE)
+  ok(true)
 end
 
-done = TRUE
+done = true
 loop{
   break
-  done = FALSE
+  done = false
 }
 ok(done)
 
-done = FALSE
-$bad = FALSE
+done = false
+$bad = false
 loop {
   break if done
-  done = TRUE
+  done = true
   next
-  $bad = TRUE
+  $bad = true
 }
 ok(!$bad)
 
-done = FALSE
-$bad = FALSE
+done = false
+$bad = false
 loop {
   break if done
-  done = TRUE
+  done = true
   redo
-  $bad = TRUE
+  $bad = true
 }
 ok(!$bad)
 
@@ -414,11 +459,11 @@ end
 ok($x.size == 7)
 ok($x == [1, 2, 3, 4, 5, 6, 7])
 
-$done = FALSE
+$done = false
 $x = []
 for i in 1 .. 7			# see how retry works in iterator loop
   if i == 4 and not $done
-    $done = TRUE
+    $done = true
     retry
   end
   $x.push(i)
@@ -426,6 +471,19 @@ end
 ok($x.size == 10)
 ok($x == [1, 2, 3, 1, 2, 3, 4, 5, 6, 7])
 
+# append method to built-in class
+class Array
+  def iter_test1
+    collect{|e| [e, yield(e)]}.sort{|a,b|a[1]<=>b[1]}
+  end
+  def iter_test2
+    a = collect{|e| [e, yield(e)]}
+    a.sort{|a,b|a[1]<=>b[1]}
+  end
+end
+$x = [[1,2],[3,4],[5,6]]
+ok($x.iter_test1{|x|x} == $x.iter_test2{|x|x})
+
 check "bignum"
 def fact(n)
   return 1 if n == 0
@@ -451,33 +509,35 @@ ok($x == -815915283247897734345611269596115894272000000000)
 ok(2-(2**32) == -(2**32-2))
 ok(2**32 - 5 == (2**32-3)-2)
 
-$good = TRUE;
-for i in 1000..1024
-  $good = FALSE if ((1<<i) != (2**i))
+$good = true;
+for i in 1000..1014
+  $good = false if ((1<<i) != (2**i))
 end
 ok($good)
 
-$good = TRUE;
+$good = true;
 n1=1<<1000
-for i in 1000..1024
-  $good = FALSE if ((1<<i) != n1)
+for i in 1000..1014
+  $good = false if ((1<<i) != n1)
   n1 *= 2
 end
 ok($good)
 
-$good = TRUE;
+$good = true;
 n2=n1
 for i in 1..10
   n1 = n1 / 2
   n2 = n2 >> 1
-  $good = FALSE if (n1 != n2)
+  $good = false if (n1 != n2)
 end
 ok($good)
 
-$good = TRUE;
-for i in 4000..4192
+$good = true;
+for i in 4000..4096
   n1 = 1 << i;
-  $good = FALSE if ((n1**2-1) / (n1+1) != (n1-1))
+  if (n1**2-1) / (n1+1) != (n1-1)
+    $good = false
+  end
 end
 ok($good)
 
@@ -486,13 +546,21 @@ check "string & char"
 ok("abcd" == "abcd")
 ok("abcd" =~ "abcd")
 ok("abcd" === "abcd")
-ok(("abc" =~ /^$/) == FALSE)
-ok(("abc\n" =~ /^$/) == FALSE)
-ok(("abc" =~ /^d*$/) == FALSE)
+ok("abc" !~ /^$/)
+ok("abc\n" !~ /^$/)
+ok("abc" !~ /^d*$/)
 ok(("abc" =~ /d*$/) == 3)
 ok("" =~ /^$/)
 ok("\n" =~ /^$/)
 ok("a\n\n" =~ /^$/)
+"abcabc" =~ /.*a/
+ok($& == "abca")
+"abcabc" =~ /.*c/
+ok($& == "abcabc")
+"abcabc" =~ /.*?a/
+ok($& == "a")
+"abcabc" =~ /.*?c/
+ok($& == "abc")
 
 $foo = "abc"
 ok("#$foo = abc" == "abc = abc")
@@ -523,13 +591,20 @@ ok(?a == ?a)
 ok(?\C-a == 1)
 ok(?\M-a == 225)
 ok(?\M-\C-a == 129)
+ok("a".upcase![0] == ?A)
+ok("A".downcase![0] == ?a)
+ok("abc".tr!("a-z", "A-Z") == "ABC")
+ok("aabbcccc".tr_s!("a-z", "A-Z") == "ABC")
+ok("abc".tr!("0-9", "A-Z") == nil)
+ok("abcc".squeeze!("a-z") == "abc")
+ok("abcd".delete!("bc") == "ad")
 
 $x = "abcdef"
 $y = [ ?a, ?b, ?c, ?d, ?e, ?f ]
-$bad = FALSE
+$bad = false
 $x.each_byte {|i|
   if i != $y.shift
-    $bad = TRUE
+    $bad = true
     break
   end
 }
@@ -556,6 +631,9 @@ ok(a == 1 && b == [2, 3])
 *a = 1, 2, 3
 ok(a == [1, 2, 3])
 
+*a = 1..3
+ok(a == [1, 2, 3])
+
 check "call"
 def aaa(a, b=100, *rest)
   res = [a, b]
@@ -566,38 +644,20 @@ end
 # not enough argument
 begin
   aaa()				# need at least 1 arg
-  ok(FALSE)
+  ok(false)
 rescue
-  ok(TRUE)
+  ok(true)
 end
 
 begin
   aaa				# no arg given (exception raised)
-  ok(FALSE)
+  ok(false)
 rescue
-  ok(TRUE)
-end
-
-begin
-  if aaa(1) == [1, 100]
-    ok(TRUE)
-  else
-    fail
-  end
-rescue
-  ok(FALSE)
-end
-
-begin
-  if aaa(1, 2) == [1, 2]
-    ok(TRUE)
-  else
-    fail
-  end
-rescue
-  ok(FALSE)
+  ok(true)
 end
 
+ok(aaa(1) == [1, 100])
+ok(aaa(1, 2) == [1, 2])
 ok(aaa(1, 2, 3, 4) == [1, 2, 3, 4])
 ok(aaa(1, *[2, 3, 4]) == [1, 2, 3, 4])
 
@@ -611,14 +671,14 @@ ok($proc.call(2) == 4)
 ok($proc.call(3) == 6)
 
 proc{
-  iii=5				# dynamic local variable
+  iii=5				# nested local variable
   $proc = proc{|i|
     iii = i
   }
   $proc2 = proc {
-    $x = iii			# dynamic variables shared by procs
+    $x = iii			# nested variables shared by procs
   }
-  # scope of dynamic variables
+  # scope of nested variables
   ok(defined?(iii))
 }.call
 ok(!defined?(iii))		# out of scope
@@ -639,35 +699,38 @@ if defined? Process.kill
 
   trap "SIGINT", proc{fail "Interrupt"}
 
-  x = FALSE
+  x = false
   begin
     Process.kill "SIGINT", $$
     sleep 0.1
   rescue
     x = $!
   end
-  ok(x && x =~ /Interrupt/)
-else
-  ok(FALSE)
+  ok(x && /Interrupt/ =~ x)
 end
 
 check "eval"
 ok(eval("") == nil)
-$bad=FALSE
-eval 'while FALSE; $bad = TRUE; print "foo\n" end'
+$bad=false
+eval 'while false; $bad = true; print "foo\n" end'
 ok(!$bad)
 
 ok(eval('TRUE'))
+ok(eval('true'))
+ok(!eval('NIL'))
+ok(!eval('nil'))
+ok(!eval('FALSE'))
+ok(!eval('false'))
 
-$foo = 'ok(TRUE)'
+$foo = 'ok(true)'
 begin
   eval $foo
 rescue
-  ok(FALSE)
+  ok(false)
 end
 
-ok(eval("$foo") == 'ok(TRUE)')
-ok(eval("TRUE") == TRUE)
+ok(eval("$foo") == 'ok(true)')
+ok(eval("true") == true)
 i = 5
 ok(eval("i == 5"))
 ok(eval("i") == 5)
@@ -683,13 +746,13 @@ def test_ev
 end
 
 $x = test_ev
-ok(eval("local1", $x) == "local1") # static local var
-ok(eval("local2", $x) == "local2") # dynamic local var
-$bad = TRUE
+ok(eval("local1", $x) == "local1") # normal local var
+ok(eval("local2", $x) == "local2") # nested local var
+$bad = true
 begin
   p eval("local1")
 rescue NameError		# must raise error
-  $bad = FALSE
+  $bad = false
 end
 ok(!$bad)
 
@@ -700,14 +763,59 @@ module EvTest
 end
 ok(eval("EVTEST1", $x) == 25)	# constant in module
 ok(eval("evtest2", $x) == 125)	# local var in module
-$bad = TRUE
+$bad = true
 begin
   eval("EVTEST1")
 rescue NameError		# must raise error
-  $bad = FALSE
+  $bad = false
 end
 ok(!$bad)
 
+x = proc{}
+eval "i4 = 1", x
+ok(eval("i4", x) == 1)
+x = proc{proc{}}.call
+eval "i4 = 22", x
+ok(eval("i4", x) == 22)
+$x = []
+x = proc{proc{}}.call
+eval "(0..9).each{|i5| $x[i5] = proc{i5*2}}", x
+ok($x[4].call == 8)
+
+x = binding
+eval "i = 1", x
+ok(eval("i", x) == 1)
+x = proc{binding}.call
+eval "i = 22", x
+ok(eval("i", x) == 22)
+$x = []
+x = proc{binding}.call
+eval "(0..9).each{|i5| $x[i5] = proc{i5*2}}", x
+ok($x[4].call == 8)
+
+proc {
+  p = binding
+  eval "foo11 = 1", p
+  proc{foo11=22}.call
+  ok(eval("foo11", p) == eval("foo11"))
+  ok(eval("foo11") == 1)
+}.call
+
+p1 = proc{i6 = 0; proc{i6}}.call
+ok(p1.call == 0)
+eval "i6=5", p1
+ok(p1.call == 5)
+ok(!defined?(i6))
+
+p1 = proc{i6 = 0; proc{i6}}.call
+i6 = nil
+ok(p1.call == 0)
+eval "i6=1", p1
+ok(p1.call == 1)
+eval "i6=5", p1
+ok(p1.call == 5)
+ok(i6 == nil)
+
 check "system"
 ok(`echo foobar` == "foobar\n")
 ok(`./ruby -e 'print "foobar"'` == 'foobar')
@@ -716,7 +824,7 @@ tmp = open("script_tmp", "w")
 tmp.print "print $zzz\n";
 tmp.close
 
-ok(`./ruby -s script_tmp -zzz` == 'TRUE')
+ok(`./ruby -s script_tmp -zzz` == 'true')
 ok(`./ruby -s script_tmp -zzz=555` == '555')
 
 tmp = open("script_tmp", "w")
@@ -744,11 +852,11 @@ end
 tmp.close
 
 `./ruby -i.bak -pe 'sub(/^[0-9]+$/){$&.to_i * 5}' script_tmp`
-done = TRUE
+done = true
 tmp = open("script_tmp", "r")
 while tmp.gets
   if $_.to_i % 5 != 0
-    done = FALSE
+    done = false
     break
   end
 end
@@ -758,6 +866,14 @@ ok(done)
 File.unlink "script_tmp" or `/bin/rm -f "script_tmp"`
 File.unlink "script_tmp.bak" or `/bin/rm -f "script_tmp.bak"`
 
+$bad = false
+for script in Dir["{lib,sample}/*.rb"]
+  unless `./ruby -c #{script}`.chomp == "Syntax OK"
+    $bad = true
+  end
+end
+ok(!$bad)
+
 check "const"
 TEST1 = 1
 TEST2 = 2
@@ -796,11 +912,16 @@ ok(foo.test == "test")
 
 begin
   foo.test2
-  ok FALSE
+  ok false
 rescue
-  ok TRUE
+  ok true
 end
 
+check "marshal"
+$x = [1,2,3,[4,5,"foo"],{1=>"bar"},2.5,fact(30)]
+$y = Marshal.dump($x)
+ok($x == Marshal.load($y))
+
 check "pack"
 
 $format = "c2x5CCxsdila6";
@@ -843,9 +964,9 @@ ok($$.instance_of?(Fixnum))
 # read-only variable
 begin
   $$ = 5
-  ok FALSE
+  ok false
 rescue
-  ok TRUE
+  ok true
 end
 
 foobar = "foobar"
@@ -931,9 +1052,9 @@ begin
     tmp = [0,1,2,3,4,5,6,7,8,9]
   }
   tmp = nil
-  ok TRUE
+  ok true
 rescue
-  ok FALSE
+  ok false
 end
 
 if $failed > 0
